/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                              Challenges Demo                             */
/*                    Populating Non-Partitioned DB with Data               */
/****************************************************************************/

/****************************************************************************/
/*  This script generates 10+ GB database and introduce heavy I/O activity  */
/****************************************************************************/


set xact_abort on
set nocount on
go

use NonPartitionedDB
go

if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name='Orders') drop table dbo.Orders;


create table dbo.Orders
(
	OrderDate datetime2(0) not null,
	OrderId int identity(1,1) not null,
	StoreId int not null,	
	CustomerID int not null, 
	Region char(5) not null,
	Amount money not null,
	Placeholder char(50) null
		constraint DEF_Placeholder
		default NEWID()
);

declare
	@FirstDate datetime2(0) = '2016-01-01'
	,@LastDate datetime2(0) = '2016-11-01'
	,@OrdersPerDay int = 200000
	,@MaxStores int = 20
	,@Mins int
	,@StartDate datetime2(0)
	,@StopDate datetime2(0)
	,@I int = 1


while @FirstDate < @LastDate
begin
	select @StopDate = dateadd(day,-1,dateadd(month,1,@FirstDate))
	set @Mins = datediff(minute,@FirstDate,@StopDate)
	;with Days(ADay)
	as
	(
		select @FirstDate
		union all
		select dateadd(day,1,ADay) from Days where ADay < @StopDate
	)
	,Data(OrderDate, Num)
	as
	(
		select dateadd(second, n.Number * 24. * 60. * 60 / @OrdersPerDay, d.ADay), 
			row_number() over (order by (select null)) 
		from Days d cross join dbo.Numbers n
		where n.Number < @OrdersPerDay
	) 
	insert into dbo.Orders with (tablockx) (OrderDate,StoreId,CustomerId,Amount,Region)
		select OrderDate, Num % @MaxStores + 1,Num % 200000, Num % 100 + .99, convert(char(5),Num % 10000) 
		from Data

	checkpoint;
	waitfor delay '00:00:05.000';
	raiserror('%d is done',0,1,@I) with nowait;
	select @FirstDate = dateadd(month,1,@FirstDate), @I += 1
end
go

update top (99.9) percent dbo.Orders with (tablockx)
set OrderDate = dateadd(month,-1,OrderDate)
where StoreId = 10 and OrderDate >= '2016-10-01'
go

-- Making more values in CI Histogram
update o with (tablockx)
set OrderDate = dateadd(hour,t.Hours,dateadd(day,t.Days,o.OrderDate))
from dbo.Orders o cross apply
	(
		select 
			case datepart(dw,o.OrderDate)
				when 7 then -1 - o.OrderId % 2
				when 0 then 1 + o.OrderId % 2
				else 0
			end as Days
			,case 
				when datepart(hour,o.OrderDate) between 18 and 23 then -6
				when datepart(hour,o.OrderDate) between 0 and 6 then 8
				else 0
			end as Hours
	) t
where 
	o.OrderID % 100 < 80 and OrderDate > '2016-01-03'
go

create clustered index IDX_Orders_OrderDate_OrderId
on dbo.Orders(OrderDate, OrderId);

create index IDX_Orders_StoreID_OrderDate 
on dbo.Orders(StoreId, OrderDate)
go

-- Let's shrink files to make DB smaller on disk. Do not run on production db :)
declare
    @MaxFileSizeMB int
    ,@SQL nvarchar(max)

-- Obtaining the current file size 	
select @MaxFileSizeMB = size / 128.0 - convert(int,fileproperty(name,'SpaceUsed')) / 128.0
from sys.database_files
where name = 'NonPartitionedDB';

set @SQL = N'dbcc shrinkfile(N''NonPartitionedDB'',' + convert(nvarchar(32),@MaxFileSizeMB) + N');';
exec sp_executesql @SQL;
go

checkpoint;
dbcc shrinkfile (N'NonPartitionedDB_log' , 16)
go

alter database NonPartitionedDB set recovery full
go