/****************************************************************************/
/*            Data Partitioning for DB Architects and Mere Mortals          */
/*                            PASS Summit 2016                              */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                              Challenges Demo                             */
/*                      Populating Partitioned DB with Data                 */
/****************************************************************************/

set noexec off
set xact_abort on
set nocount on
go

use PartitionedDB
go

if not exists(select * from sys.databases where name = 'NonPartitionedDB')
begin
	raiserror('Please create NonPartitionedDB with the data first',16,1)
	set noexec on
end
go

if not exists((select * from NonPartitionedDB.sys.tables t join NonPartitionedDB.sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name='Orders'))
begin
	raiserror('Please create PartitionedDB.Orders table first',16,1)
	set noexec on
end
go

if exists(select * from sys.views v join sys.schemas s on v.schema_id = s.schema_id where s.name = 'dbo' and v.name='Orders') drop view dbo.Orders;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name='Orders2016_10') drop table dbo.Orders2016_10;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name='Orders2016_09') drop table dbo.Orders2016_09;
if exists(select * from sys.tables t join sys.schemas s on t.schema_id = s.schema_id where s.name = 'dbo' and t.name='Orders2016') drop table dbo.Orders2016;
if exists(select * from sys.partition_schemes where name = 'psOrders2016') drop partition scheme psOrders2016;
if exists(select * from sys.partition_functions where name = 'pfOrders2016') drop partition function pfOrders2016;

create table dbo.Orders2016_10
(
	OrderDate datetime2(0) not null,
	OrderId int identity(1,1) not null,
	StoreId int not null,	
	CustomerID int not null, 
	Region char(5) not null,
	Amount money not null,
	Placeholder char(50) null,
	constraint CHK_Orders_2016_10
	check (OrderDate >= '2016-10-01' and OrderDate < '2016-11-01')
) on [RecentOrders];

create unique clustered index IDX_Orders2016_10_OrderDate_OrderId
on dbo.Orders2016_10(OrderDate,OrderId)
on [RecentOrders]
go

create table dbo.Orders2016_09
(
	OrderDate datetime2(0) not null,
	OrderId int not null,
	StoreId int not null,	
	CustomerID int not null, 
	Region char(5) not null,
	Amount money not null,
	Placeholder char(50) null,
	constraint CHK_Orders_2016_09
	check (OrderDate >= '2016-09-01' and OrderDate < '2016-10-01')
) on [RecentOrders];

create unique clustered index IDX_Orders2016_09_OrderDate_OrderId
on dbo.Orders2016_09(OrderDate,OrderId)
with (data_compression=page)
on [OldOrders]
go

create partition function pfOrders2016(datetime2(0))
as range right for values
('2016-02-01','2016-03-01','2016-04-01','2016-05-01'
,'2016-06-01','2016-07-01','2016-08-01','2016-09-01'
,'2016-10-01','2016-11-01','2016-12-01','2017-01-01');

create partition scheme psOrders2016 
as partition pfOrders2016
all to ([OldOrders]);
go

create table dbo.Orders2016
(
	OrderDate datetime2(0) not null,
	OrderId int not null,
	StoreId int not null,	
	CustomerID int not null, 
	Region char(5) not null,
	Amount money not null,
	Placeholder char(50) null,
	constraint CHK_Orders_2016
	check (OrderDate >= '2016-01-01' and OrderDate < '2016-09-01')
) on psOrders2016(OrderDate);

set identity_insert dbo.Orders2016_10 on
insert into dbo.Orders2016_10 with (TABLOCKX)
		(OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder)
	select OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder
	from NonPartitionedDB.dbo.Orders with (TABLOCK)
	where OrderDate >= '2016-10-01'
set identity_insert dbo.Orders2016_10 off
go
 
insert into dbo.Orders2016_09 with (TABLOCKX)
		(OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder)
	select OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder
	from NonPartitionedDB.dbo.Orders with (TABLOCK)
	where OrderDate >= '2016-09-01' and OrderDate < '2016-10-01';


insert into dbo.Orders2016 with (TABLOCKX)
		(OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder)
	select OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder
	from NonPartitionedDB.dbo.Orders with (TABLOCK)
	where OrderDate < '2016-09-01';

create clustered columnstore index IDX_Orders2016 on dbo.Orders2016
on psOrders2016(OrderDate)
go

create nonclustered index IDX_Orders2016_10_StoreID_OrderDate 
on dbo.Orders2016_10(StoreId, OrderDate)
on [RecentOrders]

create nonclustered index IDX_Orders2016_09_StoreID_OrderDate 
on dbo.Orders2016_09(StoreId, OrderDate)
with (data_compression=page)
on [OldOrders]
go


update statistics dbo.Orders2016_10 with fullscan;
update statistics dbo.Orders2016_09 with fullscan;
go

create view dbo.Orders(OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder)
as
	select OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder
	from dbo.Orders2016_10

	union all

	select OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder
	from dbo.Orders2016_09

	union all

	select OrderDate, OrderId, StoreId, CustomerId, Region, Amount, Placeholder
	from dbo.Orders2016
go

-- Let's shrink files to make DB smaller on disk. Do not run on production db :)
declare
    @MaxFileSizeMB int
    ,@SQL nvarchar(max)

-- Obtaining the current file size 	
select @MaxFileSizeMB = size / 128.0 - convert(int,fileproperty(name,'SpaceUsed')) / 128.0
from sys.database_files
where name = 'PartitionedDB_RecentOrders';

set @SQL = N'dbcc shrinkfile(N''PartitionedDB_RecentOrders'',' + convert(nvarchar(32),@MaxFileSizeMB) + N');';
exec sp_executesql @SQL;

select @MaxFileSizeMB = size / 128.0 - convert(int,fileproperty(name,'SpaceUsed')) / 128.0
from sys.database_files
where name = 'PartitionedDB_OldOrders';

set @SQL = N'dbcc shrinkfile(N''PartitionedDB_OldOrders'',' + convert(nvarchar(32),@MaxFileSizeMB) + N');';
exec sp_executesql @SQL;
go

checkpoint;
dbcc shrinkfile (N'PartitionedDB_log' , 16)
go


alter database PartitionedDB set recovery full
go